"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var BasicModel_1 = require("./BasicModel");
var createReduxStore_1 = require("../../src/stores/createReduxStore");
var model;
beforeAll(function () {
    createReduxStore_1.createReduxStore({});
});
beforeEach(function () {
    model = new BasicModel_1.BasicModel(Math.random().toString());
});
test('Model has right initialized reducer data', function () {
    expect(model.data).toEqual({
        id: 1,
        name: 'init-name',
    });
    expect(model.data.age).toBeUndefined();
});
test('Modify data by normal action', function () {
    expect(model.data.id).toBe(1);
    model.modify({ id: 3 });
    expect(model.data.id).toBe(3);
    model.modify({ name: 'cool' });
    expect(model.data.name).toBe('cool');
});
test('Normal action has successType', function () {
    expect(model.modify.getSuccessType()).toContain(model.constructor.name);
    expect(model.modify.getSuccessType()).toContain('modify');
    expect(model.modify.getSuccessType()).toContain('success');
});
test('Request action has three kind of types', function () {
    expect(model.getProfile.getPrepareType()).toContain(model.constructor.name);
    expect(model.getProfile.getSuccessType()).toContain(model.constructor.name);
    expect(model.getProfile.getFailType()).toContain(model.constructor.name);
    expect(model.getProfile.getPrepareType()).toContain('getProfile');
    expect(model.getProfile.getSuccessType()).toContain('getProfile');
    expect(model.getProfile.getFailType()).toContain('getProfile');
    expect(model.getProfile.getPrepareType()).toContain('prepare');
    expect(model.getProfile.getSuccessType()).toContain('success');
    expect(model.getProfile.getFailType()).toContain('fail');
});
test('Easy to use changeReducer by non-action method', function () {
    model.modifyByMethod(10002);
    expect(model.data.id).toBe(10002);
    model.modifyByMethod(200);
    expect(model.data.id).toBe(200);
    model.modifyByMethod(230);
    expect(model.data.id).toBe(230);
});
test('Reducer effect can return new object', function () {
    expect(model.data.id).toBe(1);
    model.returnNewObject();
    expect(model.data.id).toBe(100);
    expect(model.data.name).toBe('peter');
});
test('One model can have many instance', function () {
    var modelInner = new BasicModel_1.BasicModel('my-alias');
    expect(modelInner.modify.getSuccessType()).toContain('my-alias');
});
test('Even if it can be register automatically, but we can register model again and again', function () {
    expect(model.data.id).toBe(1);
    model.modify({
        id: 44,
    });
    expect(model.data.id).toBe(44);
    createReduxStore_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
    expect(model.data.id).toBe(44);
    createReduxStore_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
    expect(model.data.id).toBe(44);
});
test('Not allowed to use methods which have prefix useXXX() outside hooks-style component', function () {
    expect(function () { return model.useData(); }).toThrowError();
});
test('User can get redux data when changing reducer', function () {
    expect(function () { return model.allowGetData(); }).not.toThrowError();
});
