"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var BasicModel_1 = require("./BasicModel");
var EffectModel_1 = require("./EffectModel");
var ApiService_1 = require("./ApiService");
var src_1 = require("../../src");
var RequestModel_1 = require("./RequestModel");
var model;
beforeEach(function () {
    model = new EffectModel_1.EffectModel(Math.random().toString());
    src_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
});
test('Effect by normal action', function () {
    expect(model.data.counter).toBe(0);
    BasicModel_1.basicModel.effectOtherModel();
    expect(model.data.counter).toBe(1);
    BasicModel_1.basicModel.effectOtherModel();
    expect(model.data.counter).toBe(2);
    BasicModel_1.basicModel.effectOtherModel();
    expect(model.data.counter).toBe(3);
});
test('Effect data with payload', function () {
    expect(model.data.counter).toBe(0);
    BasicModel_1.basicModel.effectWithPayload({ counter: 130 });
    expect(model.data.counter).toBe(130);
    BasicModel_1.basicModel.effectWithPayload({ counter: 387 });
    expect(model.data.counter).toBe(387);
});
describe('Effect data by request action', function () {
    test('case success', function (done) {
        var counter = model.data.counter;
        expect(counter).toBe(0);
        ApiService_1.$api.mockResolveValue();
        var promise = RequestModel_1.requestModel.getNpmInfo('redux');
        expect(model.data.counter).toBe(counter + 5);
        promise.then(function () {
            expect(model.data.counter).toBe(counter + 5 + 7);
            done();
        });
    });
    test('case fail', function (done) {
        var counter = model.data.counter;
        expect(counter).toBe(0);
        ApiService_1.$api.mockRejectValue();
        var promise = RequestModel_1.requestModel.getNpmInfo('redux');
        expect(model.data.counter).toBe(counter + 5);
        promise
            .catch(function () {
            expect(model.data.counter).toBe(counter + 5 + 10);
            done();
        });
    });
});
