"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ManualRegisterModel_1 = require("./ManualRegisterModel");
var createReduxStore_1 = require("../../src/stores/createReduxStore");
var model;
beforeEach(function () {
    createReduxStore_1.createReduxStore({});
});
beforeEach(function () {
    model = new ManualRegisterModel_1.ManualRegisterModel(Math.random().toString());
});
test('No reducer data when we forget register', function () {
    expect(function () { return model.data.foo; }).toThrowError();
});
test('Not allowed to modify reducer when we forget register', function () {
    model.modify();
    expect(function () { return model.data.foo; }).toThrowError();
});
test('Null point register is useless', function () {
    model.register();
    expect(function () { return model.data.foo; }).toThrowError();
});
test('Reducer data can be found after register model to store', function () {
    createReduxStore_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
    expect(model.data.foo).toBe('foo');
    model.modify();
    expect(model.data.foo).toBe('bar');
    createReduxStore_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
    expect(model.data.foo).toBe('bar');
});
test('Not allowed to change reducer before register', function () {
    expect(function () { return model.testChangeReducer(''); }).toThrowError();
    createReduxStore_1.createReduxStore({
        reducers: tslib_1.__assign({}, model.register()),
    });
    expect(model.data.foo).toBe('foo');
    expect(function () { return model.testChangeReducer('new-bar'); }).not.toThrowError();
    expect(model.data.foo).toBe('new-bar');
    model.testChangeReducer('new-foo');
    expect(model.data.foo).toBe('new-foo');
});
