"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var sleep_promise_1 = tslib_1.__importDefault(require("sleep-promise"));
var PersistModel_1 = require("./PersistModel");
var createReduxStore_1 = require("../../src/stores/createReduxStore");
var core_1 = require("../../src/core");
var model;
var persistKey;
beforeEach(function () {
    persistKey = Math.random().toString();
    model = new PersistModel_1.PersistModel(Math.random().toString());
});
describe('Persist bootstrap by async', function () {
    test('localstorage', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    createReduxStore_1.createReduxStore({
                        reducers: tslib_1.__assign({}, model.register()),
                        persist: {
                            version: 1,
                            key: persistKey,
                            storage: 'local',
                            allowlist: {},
                        },
                    });
                    expect(core_1.storeHelper.persist.isReady()).toBeFalsy();
                    return [4 /*yield*/, sleep_promise_1.default(10)];
                case 1:
                    _a.sent();
                    expect(core_1.storeHelper.persist.isReady()).toBeTruthy();
                    return [2 /*return*/];
            }
        });
    }); });
    test('sessionStorage', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    createReduxStore_1.createReduxStore({
                        reducers: tslib_1.__assign({}, model.register()),
                        persist: {
                            version: 1,
                            key: persistKey,
                            storage: 'session',
                            allowlist: {},
                        },
                    });
                    expect(core_1.storeHelper.persist.isReady()).toBeFalsy();
                    return [4 /*yield*/, sleep_promise_1.default(10)];
                case 1:
                    _a.sent();
                    expect(core_1.storeHelper.persist.isReady()).toBeTruthy();
                    return [2 /*return*/];
            }
        });
    }); });
    test('memoryStorage', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    createReduxStore_1.createReduxStore({
                        reducers: tslib_1.__assign({}, model.register()),
                        persist: {
                            version: 1,
                            key: persistKey,
                            storage: 'memory',
                            allowlist: {},
                        },
                    });
                    expect(core_1.storeHelper.persist.isReady()).toBeFalsy();
                    return [4 /*yield*/, sleep_promise_1.default(10)];
                case 1:
                    _a.sent();
                    expect(core_1.storeHelper.persist.isReady()).toBeTruthy();
                    return [2 /*return*/];
            }
        });
    }); });
});
test('Restore persist data from storage', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                localStorage.setItem("ReduxModel:Persist:" + persistKey, '{"model":"{\\"counter\\":20}","__persist":{"version":1}}');
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 1,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {
                            model: model,
                        },
                    },
                });
                expect(model.data.counter).toBe(0);
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(model.data.counter).toBe(20);
                model.increase();
                expect(model.data.counter).toBe(21);
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 2:
                _a.sent();
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toContain('"model":"{\\"counter\\":21}"');
                model.increase();
                expect(model.data.counter).toBe(22);
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 3:
                _a.sent();
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toContain('"model":"{\\"counter\\":22}"');
                return [2 /*return*/];
        }
    });
}); });
test('Clear the persist data when the json data invalid', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var spy;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                localStorage.setItem("ReduxModel:Persist:" + persistKey, '{}}}}}}}');
                spy = jest.spyOn(console, 'error').mockImplementation(function () { });
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 2,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {
                            model: model,
                        },
                    },
                });
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(spy).toHaveBeenCalledTimes(1);
                expect(model.data.counter).toBe(0);
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":2}}');
                spy.mockRestore();
                return [2 /*return*/];
        }
    });
}); });
test('Clear the persist data when version is not matched', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                localStorage.setItem("ReduxModel:Persist:" + persistKey, '{"model":"{\\"counter\\":2}","__persist":{"version":1}}');
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 3,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {
                            model: model,
                        },
                    },
                });
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(model.data.counter).toBe(0);
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":3}}');
                return [2 /*return*/];
        }
    });
}); });
test('No persist data in storage', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 10,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {
                            model: model,
                        },
                    },
                });
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(model.data.counter).toBe(0);
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":10}}');
                return [2 /*return*/];
        }
    });
}); });
test('Reducer data is not hit', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                localStorage.setItem("ReduxModel:Persist:" + persistKey, '{"__persist":{"version":1}}');
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 1,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {
                            model: model,
                        },
                    },
                });
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(model.data.counter).toBe(0);
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":1}}');
                model.increase();
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 2:
                _a.sent();
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toContain('"model":"{\\"counter\\":1}"');
                return [2 /*return*/];
        }
    });
}); });
test('Restore data from storage without whitelist', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                localStorage.setItem("ReduxModel:Persist:" + persistKey, '{"model":"{\\"counter\\":2}","__persist":{"version":1}}');
                createReduxStore_1.createReduxStore({
                    reducers: tslib_1.__assign({}, model.register()),
                    persist: {
                        version: 1,
                        key: persistKey,
                        storage: 'local',
                        allowlist: {},
                    },
                });
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 1:
                _a.sent();
                expect(model.data.counter).toBe(0);
                model.increase();
                expect(model.data.counter).toBe(1);
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 2:
                _a.sent();
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":1}}');
                model.increase();
                expect(model.data.counter).toBe(2);
                return [4 /*yield*/, sleep_promise_1.default(10)];
            case 3:
                _a.sent();
                expect(localStorage.getItem("ReduxModel:Persist:" + persistKey)).toBe('{"__persist":{"version":1}}');
                return [2 /*return*/];
        }
    });
}); });
