"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ApiService_1 = require("./ApiService");
var RequestModel_1 = require("./RequestModel");
var method_1 = require("../../src/core/utils/method");
var createReduxStore_1 = require("../../src/stores/createReduxStore");
var data = {
    id: 1463,
    name: 'Join',
    age: 30,
};
var model;
beforeAll(function () {
    createReduxStore_1.createReduxStore({});
});
beforeEach(function () {
    model = new RequestModel_1.RequestModel(Math.random().toString());
});
test('Profile can be fetch by request action', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var profile;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue(data);
                return [4 /*yield*/, model.getProfile()];
            case 1:
                profile = _a.sent();
                expect(profile.response).toBe(data);
                expect(model.data).toStrictEqual(tslib_1.__assign(tslib_1.__assign({}, data), { records: model.data.records }));
                return [2 /*return*/];
        }
    });
}); });
test('Request action has the correct meta', function (done) {
    ApiService_1.$api.mockResolveValue();
    var promise = model.getProfile();
    expect(model.getProfile.loading).toBeTruthy();
    expect(model.getProfile.meta.actionType).toBe(model.getProfile.getPrepareType());
    promise.then(function (profile) {
        expect(profile.type).toBe(model.getProfile.getSuccessType());
        expect(model.getProfile.meta.actionType).toBe(model.getProfile.getSuccessType());
        expect(model.getProfile.loading).toBeFalsy();
        done();
    });
});
test('Request action has the correct metas', function (done) {
    ApiService_1.$api.mockResolveValue(data);
    var promise = model.getProfileById(data.id);
    expect(model.getProfileById.loadings.pick(data.id)).toBeTruthy();
    expect(model.getProfileById.metas.pick(data.id).actionType).toBe(model.getProfileById.getPrepareType());
    // @ts-expect-error
    expect(model.getProfileById.loading).toBeUndefined();
    // @ts-expect-error
    expect(model.getProfileById.meta.loading).toBeUndefined();
    expect(model.data.records[data.id]).toBeUndefined();
    promise.then(function (profile) {
        expect(profile.type).toBe(model.getProfileById.getSuccessType());
        expect(model.getProfileById.metas.pick(data.id).actionType).toBe(model.getProfileById.getSuccessType());
        expect(model.getProfileById.loadings.pick(data.id)).toBeFalsy();
        expect(profile.response).toBe(data);
        expect(model.data.records[data.id]).toStrictEqual(data);
        done();
    });
});
test('Fetch profile by orphan request', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var profile;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue(data);
                return [4 /*yield*/, model.orphanGetRequest()];
            case 1:
                profile = _a.sent();
                expect(profile.id).toBe(data.id);
                return [2 /*return*/];
        }
    });
}); });
test('Request not found', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var e_1;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                return [4 /*yield*/, model.getNpmInfo((new Date()).toUTCString())];
            case 1:
                _a.sent();
                return [3 /*break*/, 3];
            case 2:
                e_1 = _a.sent();
                expect(e_1.type).toBe(model.getNpmInfo.getFailType());
                expect(e_1.message).toBe('Not found');
                expect(e_1.httpStatus).toBe(404);
                return [3 /*break*/, 3];
            case 3: return [2 /*return*/];
        }
    });
}); });
test('Request timeout', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var e_2;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                _a.trys.push([0, 2, , 3]);
                return [4 /*yield*/, model.getNpmInfoWithTimeout('redux')];
            case 1:
                _a.sent();
                expect(false).toBeTruthy();
                return [3 /*break*/, 3];
            case 2:
                e_2 = _a.sent();
                expect(e_2.message).toBe('Timeout!');
                return [3 /*break*/, 3];
            case 3: return [2 /*return*/];
        }
    });
}); });
test('Easy to abort request action', function (done) {
    var promise = model.getNpmInfo('redux');
    promise
        .catch(function (e) {
        expect(e.type).toBe(model.getNpmInfo.getFailType());
        expect(e.message).toBe('Abort');
        var promise = model.getNpmInfo('react-redux');
        promise
            .catch(function (e) {
            expect(e.type).toBe(model.getNpmInfo.getFailType());
            expect(e.message).toBe('I want to cancel by myself');
            done();
        });
        promise.cancel('I want to cancel by myself');
    });
    promise.cancel();
});
describe('Request action has three kinds of reducer event', function () {
    test('case onPrepare', function () {
        ApiService_1.$api.mockResolveValue();
        model.getProfile();
        expect(model.data.id).toBe(666);
    });
    test('case onSuccess', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    ApiService_1.$api.mockResolveValue(data);
                    return [4 /*yield*/, model.getProfile()];
                case 1:
                    _a.sent();
                    expect(model.data.id).toBe(data.id);
                    return [2 /*return*/];
            }
        });
    }); });
    test('case onFail', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var _a;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    ApiService_1.$api.mockRejectValue();
                    _b.label = 1;
                case 1:
                    _b.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, model.getProfile()];
                case 2:
                    _b.sent();
                    expect(false).toBeTruthy();
                    return [3 /*break*/, 4];
                case 3:
                    _a = _b.sent();
                    expect(model.data.id).toBe(1000);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); });
});
test('Request Action has correct method', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2, result3, result4;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.getProfile()];
            case 1:
                result1 = _a.sent();
                expect(result1).toHaveProperty('method', method_1.METHOD.get);
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.withPostProfile()];
            case 2:
                result2 = _a.sent();
                expect(result2).toHaveProperty('method', method_1.METHOD.post);
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.withPutProfile()];
            case 3:
                result3 = _a.sent();
                expect(result3).toHaveProperty('method', method_1.METHOD.put);
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.withDeleteProfile()];
            case 4:
                result4 = _a.sent();
                expect(result4).toHaveProperty('method', method_1.METHOD.delete);
                return [2 /*return*/];
        }
    });
}); });
test('Request Action has correct payload', function (done) { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.payloadRequest('jack')];
            case 1:
                result1 = _a.sent();
                expect(result1.payload).toStrictEqual({
                    who: 'jack',
                });
                ApiService_1.$api.mockResolveValue();
                return [4 /*yield*/, model.payloadRequest('peter')];
            case 2:
                result2 = _a.sent();
                expect(result2.payload).toStrictEqual({
                    who: 'peter',
                });
                ApiService_1.$api.mockRejectValue();
                model.payloadRequest('boom')
                    .catch(function (e) {
                    expect(e.payload).toStrictEqual({
                        who: 'boom',
                    });
                    done();
                });
                return [2 /*return*/];
        }
    });
}); });
test('Throttle action can return remote data without real fetch', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2, result3, result4;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue({ id: 123 });
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 1:
                result1 = _a.sent();
                expect(result1.response.id).toBe(123);
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 2:
                result2 = _a.sent();
                expect(result2.response.id).toBe(123);
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 3:
                result3 = _a.sent();
                expect(result3.response.id).toBe(123);
                // Cache is expired absolutely
                return [4 /*yield*/, new Promise(function (resolve) {
                        setTimeout(resolve, 3010);
                    })];
            case 4:
                // Cache is expired absolutely
                _a.sent();
                ApiService_1.$api.mockResolveValue({ id: 987 });
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 5:
                result4 = _a.sent();
                expect(result4.response.id).toBe(987);
                return [2 /*return*/];
        }
    });
}); });
test('Throttle action always fetch remote data if the second parameter set to false', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue({ id: 123 });
                return [4 /*yield*/, model.disableCacheProfile()];
            case 1:
                result1 = _a.sent();
                expect(result1.response.id).toBe(123);
                ApiService_1.$api.mockResolveValue({ id: 456 });
                return [4 /*yield*/, model.disableCacheProfile()];
            case 2:
                result2 = _a.sent();
                expect(result2.response.id).toBe(456);
                return [2 /*return*/];
        }
    });
}); });
test('Current throttle will be removed when toggle', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2, result3;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue({ id: 123 });
                return [4 /*yield*/, model.configurableThrottle(true)];
            case 1:
                result1 = _a.sent();
                expect(result1.response.id).toBe(123);
                ApiService_1.$api.mockResolveValue({ id: 456 });
                return [4 /*yield*/, model.configurableThrottle(false)];
            case 2:
                result2 = _a.sent();
                expect(result2.response.id).toBe(456);
                ApiService_1.$api.mockResolveValue({ id: 789 });
                return [4 /*yield*/, model.configurableThrottle(true)];
            case 3:
                result3 = _a.sent();
                expect(result3.response.id).toBe(789);
                return [2 /*return*/];
        }
    });
}); });
test('Clear Throttle action by hand', function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
    var result1, result2, result3, result4;
    return tslib_1.__generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                ApiService_1.$api.mockResolveValue({ id: 123 });
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 1:
                result1 = _a.sent();
                expect(result1.response.id).toBe(123);
                // Cache
                ApiService_1.$api.mockResolveValue({ id: 666 });
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 2:
                result2 = _a.sent();
                expect(result2.response.id).toBe(123);
                model.enableThrottleProfile.clearThrottle();
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 3:
                result3 = _a.sent();
                expect(result3.response.id).toBe(666);
                return [4 /*yield*/, model.enableThrottleProfile()];
            case 4:
                result4 = _a.sent();
                expect(result4.response.id).toBe(666);
                return [2 /*return*/];
        }
    });
}); });
